VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "clsSpreadGrid"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'********************************************************************************************************
'Title:     clsSpreadGrid
'Author:    DesignGrid by W. David Ewing, Copyright 1998
'Purpose    This class was created for ease of programming with the Spread style grid controls
'           It assumes you will be using the data grid for display purposes, and will be
'           using it as a kind of list box.  This is something I have found myself doing
'           quite a bit with client server databases.
'Requires:  Data Access Class

'This is commented because it is recommended that objError be declared global
'The reason for this is so that the DisplayFlag and writeToFile properties will
'be persistent
'Private objError as new clsError

'It is recommended that the Database object be declared global

'It is also recommended that the Configuration object be declared global if it is being used
'This is so that it can be persistent
'**************************************************************************************

Public Success As Boolean
Public ErrorCode As Double
Public ErrorMessage As String
Private objDataAccess As New clsDataAccess


'********************************************************************************************************
'Title:     SetNextRecord
'Author:    DesignGrid by W. David Ewing, Copyright 1998
'Purpose:   This Sub moves to the next grid record if possible
'           It returns a true if repositioning is needed, or a false if it is not
'           It is used to position grid rows
'Parameters:The Spread control to be repositioned
'Return:    Nothing
'********************************************************************************************************
Public Sub SetNextRecord(pData As Control)

Dim licount As Integer, BadCount As Integer
Dim lsSelect As String

    'The Success flag gets initialized to True and set to false if a trappable error occurs
    Success = True
    'The ErrorCode is the Err returned by VB for the Trapped Error
    ErrorCode = False
    'The DebugFlag is the provision which turns off all error checking in the table class when false
    If Not objConfiguration.DebugFlag Then
        On Error GoTo NoSpreadGridSetNextRecord
    End If
    If pData.Rows = 0 Then
        Success = False
        On Error GoTo 0
        Exit Sub
    End If
    If pData.Row = pData.Rows Then
        Success = False
        On Error GoTo 0
        Exit Sub
    End If
    pData.Row = pData.Row + 1
    pData.Action = 0
    Success = True
    On Error GoTo 0
    Exit Sub
    
NoSpreadGridSetNextRecord:

    Success = False
    ErrorCode = Err
    objError.ErrorCode = Err
    objError.FunctionName = "clsSpreadGrid.SetNextRecord"
    If Err = 3146 Then
        objError.Message = "SpreadGrid, SetNextRecord " & vbCrLf & Errors(0) & " "
        ErrorMessage = Errors(0)
    Else
        objError.Message = "SpreadGrid, SetNextRecord "
        ErrorMessage = Error(Err)
    End If
    objError.SQL = lsSelect
    objError.Display vbExclamation
    Resume Next

    
End Sub

'********************************************************************************************************
'Title:     SetPreviousRecord
'Author:    DesignGrid by W. David Ewing, Copyright 1998
'Purpose:   This Sub moves to the Previous grid record if possible
'           It returns a true if repositioning is needed, or a false if it is not
'           It is used to position grid rows
'Parameters:The Spread control to be repositioned
'Return:    Nothing
'********************************************************************************************************
Public Sub SetPreviousRecord(pData As Control)

Dim licount As Integer, BadCount As Integer
Dim lsSelect As String

    'The Success flag gets initialized to True and set to false if a trappable error occurs
    Success = True
    'The ErrorCode is the Err returned by VB for the Trapped Error
    ErrorCode = False
    'The DebugFlag is the provision which turns off all error checking in the table class when false
    If Not objConfiguration.DebugFlag Then
        On Error GoTo NoSpreadGridSetPreviousRecord
    End If
    If pData.Rows = 0 Then
        Success = False
        On Error GoTo 0
        Exit Sub
    End If
    If pData.Row = 1 Then
        Success = False
        On Error GoTo 0
        Exit Sub
    End If
    pData.Row = pData.Row - 1
    pData.Action = 0
    Success = True
    On Error GoTo 0
    Exit Sub
    
NoSpreadGridSetPreviousRecord:

    Success = False
    ErrorCode = Err
    objError.ErrorCode = Err
    objError.FunctionName = "clsSpreadGrid.SetPreviousRecord"
    If Err = 3146 Then
        objError.Message = "SpreadGrid, SetPreviousRecord " & vbCrLf & Errors(0) & " "
        ErrorMessage = Errors(0)
    Else
        objError.Message = "SpreadGrid, SetPreviousRecord "
        ErrorMessage = Error(Err)
    End If
    objError.SQL = lsSelect
    objError.Display vbExclamation
    Resume Next

    
End Sub

'********************************************************************************************************
'Title:     SetNonDeletedRecord
'Author:    DesignGrid by W. David Ewing, Copyright 1998
'Purpose:   This Sub moves to the Previous grid record if possible if not,
'           then it will set to the next record.  This is to take care of the unique
'           situation which occurs when a record is deleted.
'           It returns a true if repositioning is needed, or a false if it is not
'           It is used to position grid rows
'Parameters:The Spread control to be repositioned
'Return:    Nothing
'********************************************************************************************************
Public Sub SetNonDeletedRecord(pData As Control)

Dim licount As Integer, BadCount As Integer
Dim lsSelect As String

    'The Success flag gets initialized to True and set to false if a trappable error occurs
    Success = True
    'The ErrorCode is the Err returned by VB for the Trapped Error
    ErrorCode = False
    'The DebugFlag is the provision which turns off all error checking in the table class when false
    If Not objConfiguration.DebugFlag Then
        On Error GoTo NoSpreadGridSetNonDeletedRecord
    End If
    If pData.Rows = 0 Then
        Success = False
        On Error GoTo 0
        Exit Sub
    End If
    If pData.Row = 1 Then
        Success = False
        pData.Row = 2
        pData.Action = 0
        On Error GoTo 0
        Exit Sub
    End If
    pData.Row = pData.Row - 1
    Success = True
    On Error GoTo 0
    Exit Sub
    
NoSpreadGridSetNonDeletedRecord:

    Success = False
    ErrorCode = Err
    objError.ErrorCode = Err
    objError.FunctionName = "clsSpreadGrid.SetNonDeletedRecord"
    If Err = 3146 Then
        objError.Message = "SpreadGrid, SetNonDeletedRecord " & vbCrLf & Errors(0) & " "
        ErrorMessage = Errors(0)
    Else
        objError.Message = "SpreadGrid, SetNonDeletedRecord "
        ErrorMessage = Error(Err)
    End If
    objError.SQL = lsSelect
    objError.Display vbExclamation
    Resume Next

    
End Sub

'********************************************************************************************************
'Title:     FillDataGrid
'Author:    DesignGrid by W. David Ewing, Copyright 1998
'Purpose:   This Function Fills a Spread control using the SQL passed to it.  If the HeaderFlag
'           is set to true then the column names of the sql statement will be used to fill the grid col captions
'Parameters:The Spread control to be filled, and header flag
'Return:    Number of Rows Found
'********************************************************************************************************
Public Function FillDataGrid(psSQL As String, pData As Control, pHeaderFlag As Integer) As Integer

Dim lrsData As Recordset
Dim lsSelect As String
Dim licount As Integer, BadCount As Integer

    'The Success flag gets initialized to True and set to false if a trappable error occurs
    Success = True
    'The ErrorCode is the Err returned by VB for the Trapped Error
    ErrorCode = False
    'The DebugFlag is the provision which turns off all error checking in the table class when false
    If Not objConfiguration.DebugFlag Then
        On Error GoTo NoSpreadGridFillDataGrid
    End If
    lsSelect = psSQL
    'Open the recordset with sql from the class
    Set lrsData = objDataAccess.OpenRecordSetWithRecs(lsSelect)

    'Check for errors
    If lrsData Is Nothing Then
        pData.Rows = 0
        FillDataGrid = 0
        Success = False
        ErrorCode = objDataAccess.ErrorCode
        On Error GoTo 0
        Exit Function
    End If
    
    'move last to get the correct record count
    lrsData.MoveLast
    'put the recordset into the data object for display in the list
    pData.Cols = lrsData.Fields.Count
    pData.Rows = lrsData.RecordCount + 1
    lrsData.MoveFirst
    pData.Row = 0
    
    'if the header flag is set, fill the headers
    If pHeaderFlag Then
        pData.Col = 0
        Do While True
            pData.Text = lrsData.Fields(pData.Col).Name
            If pData.Col = pData.Cols - 1 Then
                Exit Do
            End If
            pData.Col = pData.Col + 1
        Loop
    End If
    
    'now fill the grid
    Do
        pData.Row = pData.Row + 1
        pData.Col = 0
        Do
            pData.Text = lrsData(pData.Col) & ""
            If pData.Col = pData.Cols - 1 Then
                Exit Do
            End If
            pData.Col = pData.Col + 1
        Loop While True
        lrsData.MoveNext
    Loop While pData.Row < pData.Rows - 1
    
    FillDataGrid = pData.Rows - 1
    lrsData.Close
    On Error GoTo 0
    Exit Function

NoSpreadGridFillDataGrid:

    Success = False
    ErrorCode = Err
    objError.ErrorCode = Err
    objError.FunctionName = "clsSpreadGrid.FillDataGrid"
    If Err = 3146 Then
        objError.Message = "SpreadGrid, FillDataGrid " & vbCrLf & Errors(0) & " "
        ErrorMessage = Errors(0)
    Else
        objError.Message = "SpreadGrid, FillDataGrid "
        ErrorMessage = Error(Err)
    End If
    objError.SQL = lsSelect
    objError.Display vbExclamation
    Resume Next

    
End Function

'********************************************************************************************************
'Title:     MoveToRecord
'Author:    DesignGrid by W. David Ewing, Copyright 1998
'Purpose:   This Sub Moves the grid selection to a row specified by a keyarray.  The
'           Key array is an array of variants which the sub checks against the first values
'           in the grid.  It assumes the first column or columns in the grid are the key
'           These columns can be hidden so that the user cannot see them.  The programmer
'           should declare the array with number of keys being matched, then fill the array.
'Parameters:The Spread control to be repositioned, and the Key Array to be matched
'Return:    nothing
'********************************************************************************************************
Public Sub MoveToRecord(pData As Control, KeyArray() As Variant)

Dim MatchCounter As Integer
Dim licount As Integer, BadCount As Integer
Dim lsSelect As String

    'The Success flag gets initialized to True and set to false if a trappable error occurs
    Success = True
    'The ErrorCode is the Err returned by VB for the Trapped Error
    ErrorCode = False
    'The DebugFlag is the provision which turns off all error checking in the table class when false
    If Not objConfiguration.DebugFlag Then
        On Error GoTo NoSpreadGridMoveToRecord
    End If
    pData.Row = 0
    Do
        pData.Row = pData.Row + 1
        MatchCounter = 0
        pData.Col = 0
        For licount = 1 To UBound(KeyArray)
            pData.Col = licount - 1
            If IsNumeric(KeyArray(licount)) And IsNumeric(pData.Text) Then
                If Val(KeyArray(licount)) = Val(pData.Text) Then
                    MatchCounter = MatchCounter + 1
                End If
            Else
                If IsDate(KeyArray(licount)) And IsDate(pData.Text) Then
                    If CVDate(KeyArray(licount)) = CVDate(pData.Text) Then
                        MatchCounter = MatchCounter + 1
                    End If
                Else
                    If KeyArray(licount) = pData.Text Then
                        MatchCounter = MatchCounter + 1
                    End If
                End If
            End If
        Next
        If MatchCounter = UBound(KeyArray) Then
            On Error GoTo 0
            Exit Sub
        End If
    Loop While pData.Row < pData.Rows - 1
    pData.Col = 0
    On Error GoTo 0
    Exit Sub
    
NoSpreadGridMoveToRecord:

    Success = False
    ErrorCode = Err
    objError.ErrorCode = Err
    objError.FunctionName = "clsSpreadGrid.MoveToRecord"
    If Err = 3146 Then
        objError.Message = "SpreadGrid, MoveToRecord " & vbCrLf & Errors(0) & " "
        ErrorMessage = Errors(0)
    Else
        objError.Message = "SpreadGrid, MoveToRecord "
        ErrorMessage = Error(Err)
    End If
    objError.SQL = lsSelect
    objError.Display vbExclamation
    Resume Next

    
End Sub


'********************************************************************************************************
'Title:     ClearSelections
'Author:    DesignGrid by W. David Ewing, Copyright 1998
'Purpose:   This sub clears any and all selections on a spread grid
'Parameters:The Spread control to be cleared
'Return:    nothing
'********************************************************************************************************
Public Sub ClearSelections(pData As Control)

Dim licount As Integer, BadCount As Integer
Dim lsSelect As String

    'The Success flag gets initialized to True and set to false if a trappable error occurs
    Success = True
    'The ErrorCode is the Err returned by VB for the Trapped Error
    ErrorCode = False
    'The DebugFlag is the provision which turns off all error checking in the table class when false
    If Not objConfiguration.DebugFlag Then
        On Error GoTo NoSpreadGridClearSelections
    End If

    For licount = 1 To pData.Rows
        pData.Row = licount
        pData.SelModeSelected = False
    Next
    Exit Sub

NoSpreadGridClearSelections:

    Success = False
    ErrorCode = Err
    objError.ErrorCode = Err
    objError.FunctionName = "clsSpreadGrid.ClearSelections"
    If Err = 3146 Then
        objError.Message = "SpreadGrid, ClearSelections " & vbCrLf & Errors(0) & " "
        ErrorMessage = Errors(0)
    Else
        objError.Message = "SpreadGrid, ClearSelections "
        ErrorMessage = Error(Err)
    End If
    objError.SQL = lsSelect
    objError.Display vbExclamation
    Resume Next


End Sub

'********************************************************************************************************
'Title:     CountVisibleCols
'Author:    DesignGrid by W. David Ewing, Copyright 1998
'Purpose:   This Function looks at each column in the grid to determine how many left hand cells
'           are hidden.  This is necessary to resize the grid columns properly with the
'           column resize function
'Parameters:The Spread control to be checked
'Return:    number of left hand hidden columns
'********************************************************************************************************
Public Function CountVisibleCols(pGrid As Control) As Integer

Dim licount As Integer, BadCount As Integer
Dim lsSelect As String

    'The Success flag gets initialized to True and set to false if a trappable error occurs
    Success = True
    'The ErrorCode is the Err returned by VB for the Trapped Error
    ErrorCode = False
    'The DebugFlag is the provision which turns off all error checking in the table class when false
    If Not objConfiguration.DebugFlag Then
        On Error GoTo NoSpreadGridCountVisibleCols
    End If

    For licount = 0 To pGrid.Cols - 1
        If pGrid.ColIsVisible(licount) Then
            CountVisibleCols = CountVisibleCols + 1
        End If
    Next
    Exit Function

NoSpreadGridCountVisibleCols:

    Success = False
    ErrorCode = Err
    objError.ErrorCode = Err
    objError.FunctionName = "clsSpreadGrid.CountVisibleCols"
    If Err = 3146 Then
        objError.Message = "SpreadGrid, CountVisibleCols " & vbCrLf & Errors(0) & " "
        ErrorMessage = Errors(0)
    Else
        objError.Message = "SpreadGrid, CountVisibleCols "
        ErrorMessage = Error(Err)
    End If
    objError.SQL = lsSelect
    objError.Display vbExclamation
    Resume Next


End Function

'********************************************************************************************************
'Title:     ResizeGridCols
'Author:    DesignGrid by W. David Ewing, Copyright 1998
'Purpose:   This sub resizes the columns in a grid equally to fit its width
'           it is useful when dynamic resizing of a form is desired
'Parameters:The Spread control to be resized
'Return:    nothing
'********************************************************************************************************
Public Sub ResizeGridCols(pGrid As Control)

Dim llTwipSize As Long, liVisibleCols As Integer
Dim lsngColWidth As Single, ldFactor As Double
Dim licount As Integer, BadCount As Integer
Dim lsSelect As String

    'The Success flag gets initialized to True and set to false if a trappable error occurs
    Success = True
    'The ErrorCode is the Err returned by VB for the Trapped Error
    ErrorCode = False
    'The DebugFlag is the provision which turns off all error checking in the table class when false
    If Not objConfiguration.DebugFlag Then
        On Error GoTo NoSpreadGridResizeGridCols
    End If

    'Get the Visible Column Count
    liVisibleCols = CountVisibleCols(pGrid)
    'resize all the columns
    For licount = (pGrid.Cols - liVisibleCols) To pGrid.Cols - 1
        'Get the column size according to width and number of columns,account for the scroll bar
        llTwipSize = (pGrid.Width - 350) / liVisibleCols
        'resize the column converting from twips
        pGrid.ColWidth(licount) = llTwipSize
    Next
    Exit Sub

NoSpreadGridResizeGridCols:

    Success = False
    ErrorCode = Err
    objError.ErrorCode = Err
    objError.FunctionName = "clsSpreadGrid.ResizeGridCols"
    If Err = 3146 Then
        objError.Message = "SpreadGrid, ResizeGridCols " & vbCrLf & Errors(0) & " "
        ErrorMessage = Errors(0)
    Else
        objError.Message = "SpreadGrid, ResizeGridCols "
        ErrorMessage = Error(Err)
    End If
    objError.SQL = lsSelect
    objError.Display vbExclamation
    Resume Next


End Sub

